@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2017 Oracle Corporation
+ * Copyright (C) 2017-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -24,8 +24,11 @@
  * terms and conditions of either the GPL or the CDDL or both.
  */
 
-#ifndef ___iprt_formats_iso9660_h
-#define ___iprt_formats_iso9660_h
+#ifndef IPRT_INCLUDED_formats_iso9660_h
+#define IPRT_INCLUDED_formats_iso9660_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include <iprt/types.h>
 #include <iprt/assertcompile.h>
@@ -1497,5 +1500,5 @@ typedef ISO9660SUSPUNION *PCISO9660SUSPUNION;
 
 /** @} */
 
-#endif
+#endif /* !IPRT_INCLUDED_formats_iso9660_h */
 
