@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2016-2019 Oracle Corporation
+ * Copyright (C) 2016-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -24,32 +24,47 @@
 #include "AudioMixer.h"
 #include <VBox/log.h> /* LOG_ENABLED */
 
-/** See 302349 p 6.2. */
+/** Pointer to an HDA stream (SDI / SDO).  */
+typedef struct HDASTREAMR3 *PHDASTREAMR3;
+
+
+
+/** Read callback. */
+typedef VBOXSTRICTRC FNHDAREGREAD(PPDMDEVINS pDevIns, PHDASTATE pThis, uint32_t iReg, uint32_t *pu32Value);
+/** Write callback. */
+typedef VBOXSTRICTRC FNHDAREGWRITE(PPDMDEVINS pDevIns, PHDASTATE pThis, uint32_t iReg, uint32_t u32Value);
+
+/**
+ * HDA register descriptor.
+ *
+ * See 302349 p 6.2.
+ */
 typedef struct HDAREGDESC
 {
     /** Register offset in the register space. */
-    uint32_t    offset;
+    uint32_t        offset;
     /** Size in bytes. Registers of size > 4 are in fact tables. */
-    uint32_t    size;
+    uint32_t        size;
     /** Readable bits. */
-    uint32_t    readable;
+    uint32_t        readable;
     /** Writable bits. */
-    uint32_t    writable;
-    /** Register descriptor (RD) flags of type HDA_RD_FLAG_.
-     *  These are used to specify the handling (read/write)
-     *  policy of the register. */
-    uint32_t    fFlags;
+    uint32_t        writable;
+    /** Register descriptor (RD) flags of type HDA_RD_F_XXX. These are used to
+     *  specify the handling (read/write) policy of the register. */
+    uint32_t        fFlags;
     /** Read callback. */
-    int       (*pfnRead)(PHDASTATE pThis, uint32_t iReg, uint32_t *pu32Value);
+    FNHDAREGREAD   *pfnRead;
     /** Write callback. */
-    int       (*pfnWrite)(PHDASTATE pThis, uint32_t iReg, uint32_t u32Value);
+    FNHDAREGWRITE  *pfnWrite;
     /** Index into the register storage array. */
-    uint32_t    mem_idx;
+    uint32_t        mem_idx;
     /** Abbreviated name. */
-    const char *abbrev;
+    const char     *abbrev;
     /** Descripton. */
-    const char *desc;
-} HDAREGDESC, *PHDAREGDESC;
+    const char     *desc;
+} HDAREGDESC;
+/** Pointer to a a const HDA register descriptor. */
+typedef HDAREGDESC const *PCHDAREGDESC;
 
 /**
  * HDA register aliases (HDA spec 3.3.45).
@@ -61,7 +76,7 @@ typedef struct HDAREGALIAS
     uint32_t    offReg;
     /** The register index. */
     int         idxAlias;
-} HDAREGALIAS, *PHDAREGALIAS;
+} HDAREGALIAS;
 
 /**
  * At the moment we support 4 input + 4 output streams max, which is 8 in total.
@@ -520,15 +535,15 @@ extern const HDAREGDESC g_aHdaRegMap[HDA_NUM_REGS];
      | ( (_aChan)     & HDA_SDFMT_CHANNELS_MASK))
 
 /** Interrupt on completion (IOC) flag. */
-#define HDA_BDLE_FLAG_IOC           RT_BIT(0)
+#define HDA_BDLE_F_IOC              RT_BIT(0)
 
 
 
-/** The HDA controller. */
+/** Pointer to a shared HDA state. */
 typedef struct HDASTATE *PHDASTATE;
-/** The HDA stream. */
+/** Pointer to a HDA stream state. */
 typedef struct HDASTREAM *PHDASTREAM;
-
+/** Pointer to a mixer sink. */
 typedef struct HDAMIXERSINK *PHDAMIXERSINK;
 
 
@@ -587,22 +602,25 @@ typedef HDABDLE *PHDABDLE;
  * @{
  */
 #ifdef IN_RING3
-PHDAMIXERSINK hdaR3GetDefaultSink(PHDASTATE pThis, uint8_t uSD);
+PHDAMIXERSINK hdaR3GetDefaultSink(PHDASTATER3 pThisCC, uint8_t uSD);
 #endif
 PDMAUDIODIR   hdaGetDirFromSD(uint8_t uSD);
-PHDASTREAM    hdaGetStreamFromSD(PHDASTATE pThis, uint8_t uSD);
+//PHDASTREAM    hdaGetStreamFromSD(PHDASTATER3 pThisCC, uint8_t uSD);
 #ifdef IN_RING3
-PHDASTREAM    hdaR3GetStreamFromSink(PHDASTATE pThis, PHDAMIXERSINK pSink);
+PHDASTREAMR3  hdaR3GetR3StreamFromSink(PHDAMIXERSINK pSink);
+PHDASTREAM    hdaR3GetSharedStreamFromSink(PHDAMIXERSINK pSink);
 #endif
 /** @} */
 
 /** @name Interrupt functions.
  * @{
  */
-#ifdef LOG_ENABLED
-int           hdaProcessInterrupt(PHDASTATE pThis, const char *pszSource);
+#if defined(LOG_ENABLED) || defined(DOXYGEN_RUNNING)
+void          hdaProcessInterrupt(PPDMDEVINS pDevIns, PHDASTATE pThis, const char *pszSource);
+# define HDA_PROCESS_INTERRUPT(a_pDevIns, a_pThis)  hdaProcessInterrupt((a_pDevIns), (a_pThis), __FUNCTION__)
 #else
-int           hdaProcessInterrupt(PHDASTATE pThis);
+void          hdaProcessInterrupt(PPDMDEVINS pDevIns, PHDASTATE pThis);
+# define HDA_PROCESS_INTERRUPT(a_pDevIns, a_pThis)  hdaProcessInterrupt((a_pDevIns), (a_pThis))
 #endif
 /** @} */
 
@@ -611,7 +629,7 @@ int           hdaProcessInterrupt(PHDASTATE pThis);
  */
 uint64_t      hdaWalClkGetCurrent(PHDASTATE pThis);
 #ifdef IN_RING3
-bool          hdaR3WalClkSet(PHDASTATE pThis, uint64_t u64WalClk, bool fForce);
+bool          hdaR3WalClkSet(PHDASTATE pThis, PHDASTATER3 pThisCC, uint64_t u64WalClk, bool fForce);
 #endif
 /** @} */
 
@@ -619,8 +637,10 @@ bool          hdaR3WalClkSet(PHDASTATE pThis, uint64_t u64WalClk, bool fForce);
  * @{
  */
 #ifdef IN_RING3
-int           hdaR3DMARead(PHDASTATE pThis, PHDASTREAM pStream, void *pvBuf, uint32_t cbBuf, uint32_t *pcbRead);
-int           hdaR3DMAWrite(PHDASTATE pThis, PHDASTREAM pStream, const void *pvBuf, uint32_t cbBuf, uint32_t *pcbWritten);
+int           hdaR3DMARead(PPDMDEVINS pDevIns, PHDASTATE pThis, PHDASTREAM pStreamShared, PHDASTREAMR3 pStreamR3,
+                           void *pvBuf, uint32_t cbBuf, uint32_t *pcbRead);
+int           hdaR3DMAWrite(PPDMDEVINS pDevIns, PHDASTATE pThis, PHDASTREAM pStreamShared, PHDASTREAMR3 pStreamR3,
+                            const void *pvBuf, uint32_t cbBuf, uint32_t *pcbWritten);
 #endif
 /** @} */
 
@@ -638,9 +658,9 @@ int           hdaR3SDFMTToPCMProps(uint16_t u16SDFMT, PPDMAUDIOPCMPROPS pProps);
  */
 #ifdef IN_RING3
 # ifdef LOG_ENABLED
-void          hdaR3BDLEDumpAll(PHDASTATE pThis, uint64_t u64BDLBase, uint16_t cBDLE);
+void          hdaR3BDLEDumpAll(PPDMDEVINS pDevIns, PHDASTATE pThis, uint64_t u64BDLBase, uint16_t cBDLE);
 # endif
-int           hdaR3BDLEFetch(PHDASTATE pThis, PHDABDLE pBDLE, uint64_t u64BaseDMA, uint16_t u16Entry);
+int           hdaR3BDLEFetch(PPDMDEVINS pDevIns, PHDABDLE pBDLE, uint64_t u64BaseDMA, uint16_t u16Entry);
 bool          hdaR3BDLEIsComplete(PHDABDLE pBDLE);
 bool          hdaR3BDLENeedsInterrupt(PHDABDLE pBDLE);
 #endif /* IN_RING3 */
@@ -650,8 +670,8 @@ bool          hdaR3BDLENeedsInterrupt(PHDABDLE pBDLE);
  * @{
  */
 #ifdef IN_RING3
-bool          hdaR3TimerSet(PHDASTATE pThis, PHDASTREAM pStream, uint64_t u64Expire, bool fForce);
-#endif /* IN_RING3 */
+bool          hdaR3TimerSet(PPDMDEVINS pDevIns, PHDASTREAM pStreamShared, uint64_t u64Expire, bool fForce, uint64_t tsNow);
+#endif
 /** @} */
 
 #endif /* !VBOX_INCLUDED_SRC_Audio_DevHDACommon_h */
