@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,8 +15,11 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
-#ifndef ____H_VIRTUALBOXIMPL
-#define ____H_VIRTUALBOXIMPL
+#ifndef MAIN_INCLUDED_VirtualBoxImpl_h
+#define MAIN_INCLUDED_VirtualBoxImpl_h
+#ifndef RT_WITHOUT_PRAGMA_ONCE
+# pragma once
+#endif
 
 #include "VirtualBoxBase.h"
 #include "objectslist.h"
@@ -26,6 +29,10 @@
 # include "win/resource.h"
 #endif
 
+//#ifdef DEBUG_bird
+//# define VBOXSVC_WITH_CLIENT_WATCHER
+//#endif
+
 namespace com
 {
     class Event;
@@ -39,11 +46,15 @@ class Host;
 class SystemProperties;
 class DHCPServer;
 class PerformanceCollector;
+class CloudProviderManager;
 #ifdef VBOX_WITH_EXTPACK
 class ExtPackManager;
 #endif
 class AutostartDb;
 class NATNetwork;
+#ifdef VBOX_WITH_CLOUD_NET
+class CloudNetwork;
+#endif /* VBOX_WITH_CLOUD_NET */
 
 
 typedef std::list<ComObjPtr<SessionMachine> > SessionMachinesList;
@@ -63,6 +74,9 @@ namespace settings
 class VirtualBoxClassFactory; /* See ../src-server/win/svcmain.cpp  */
 #endif
 
+#ifdef VBOX_WITH_SHARED_CLIPBOARD_TRANSFERS
+struct SharedClipboardAreaData;
+#endif
 
 class ATL_NO_VTABLE VirtualBox :
     public VirtualBoxWrap
@@ -144,6 +158,11 @@ public:
     int i_loadVDPlugin(const char *pszPluginLibrary);
     int i_unloadVDPlugin(const char *pszPluginLibrary);
 
+    void i_onMediumRegistered(const Guid &aMediumId, const DeviceType_T aDevType, const BOOL aRegistered);
+    void i_onMediumConfigChanged(IMedium *aMedium);
+    void i_onMediumChanged(IMediumAttachment* aMediumAttachment);
+    void i_onStorageControllerChanged(const Guid &aMachineId, const com::Utf8Str &aControllerName);
+    void i_onStorageDeviceChanged(IMediumAttachment* aStorageDevice, const BOOL fRemoved, const BOOL fSilent);
     void i_onMachineStateChange(const Guid &aId, MachineState_T aState);
     void i_onMachineDataChange(const Guid &aId, BOOL aTemporary = FALSE);
     BOOL i_onExtraDataCanChange(const Guid &aId, IN_BSTR aKey, IN_BSTR aValue,
@@ -156,6 +175,19 @@ public:
     void i_onSnapshotDeleted(const Guid &aMachineId, const Guid &aSnapshotId);
     void i_onSnapshotRestored(const Guid &aMachineId, const Guid &aSnapshotId);
     void i_onSnapshotChange(const Guid &aMachineId, const Guid &aSnapshotId);
+
+#ifdef VBOX_WITH_SHARED_CLIPBOARD_TRANSFERS
+    int i_clipboardAreaCreate(ULONG uAreaID, uint32_t fFlags, SharedClipboardAreaData **ppAreaData);
+    int i_clipboardAreaDestroy(SharedClipboardAreaData *pAreaData);
+
+    HRESULT i_onClipboardAreaRegister(const std::vector<com::Utf8Str> &aParms, ULONG *aID);
+    HRESULT i_onClipboardAreaUnregister(ULONG aID);
+    HRESULT i_onClipboardAreaAttach(ULONG aID);
+    HRESULT i_onClipboardAreaDetach(ULONG aID);
+    ULONG i_onClipboardAreaGetMostRecent(void);
+    ULONG i_onClipboardAreaGetRefCount(ULONG aID);
+#endif /* VBOX_WITH_SHARED_CLIPBOARD_TRANSFERS */
+
     void i_onGuestPropertyChange(const Guid &aMachineId, IN_BSTR aName, IN_BSTR aValue,
                                  IN_BSTR aFlags);
     void i_onNatRedirectChange(const Guid &aMachineId, ULONG ulSlot, bool fRemove, IN_BSTR aName,
@@ -175,6 +207,12 @@ public:
     int i_natNetworkRefInc(const Utf8Str &aNetworkName);
     int i_natNetworkRefDec(const Utf8Str &aNetworkName);
 
+#ifdef VBOX_WITH_CLOUD_NET
+    HRESULT i_findCloudNetworkByName(const com::Utf8Str &aNetworkName,
+                                     ComObjPtr<CloudNetwork> *aNetwork = NULL);
+    HRESULT i_getEventSource(ComPtr<IEventSource>& aSource);
+#endif /* VBOX_WITH_CLOUD_NET */
+
     ComObjPtr<GuestOSType> i_getUnknownOSType();
 
     void i_getOpenedMachines(SessionMachinesList &aMachines,
@@ -215,20 +253,21 @@ public:
 
     const Guid &i_getGlobalRegistryId() const;
 
-    const ComObjPtr<Host>& i_host() const;
-    SystemProperties* i_getSystemProperties() const;
+    const ComObjPtr<Host> &i_host() const;
+    SystemProperties *i_getSystemProperties() const;
+    CloudProviderManager *i_getCloudProviderManager() const;
 #ifdef VBOX_WITH_EXTPACK
-    ExtPackManager* i_getExtPackManager() const;
+    ExtPackManager *i_getExtPackManager() const;
 #endif
 #ifdef VBOX_WITH_RESOURCE_USAGE_API
-    const ComObjPtr<PerformanceCollector>& i_performanceCollector() const;
+    const ComObjPtr<PerformanceCollector> &i_performanceCollector() const;
 #endif /* VBOX_WITH_RESOURCE_USAGE_API */
 
     void i_getDefaultMachineFolder(Utf8Str &str) const;
     void i_getDefaultHardDiskFormat(Utf8Str &str) const;
 
     /** Returns the VirtualBox home directory */
-    const Utf8Str& i_homeDir() const;
+    const Utf8Str &i_homeDir() const;
     int i_calculateFullPath(const Utf8Str &strPath, Utf8Str &aResult);
     void i_copyPathRelativeToConfig(const Utf8Str &strSource, Utf8Str &strTarget);
     HRESULT i_registerMedium(const ComObjPtr<Medium> &pMedium, ComObjPtr<Medium> *ppMedium,
@@ -288,6 +327,8 @@ private:
     HRESULT getExtensionPackManager(ComPtr<IExtPackManager> &aExtensionPackManager);
     HRESULT getInternalNetworks(std::vector<com::Utf8Str> &aInternalNetworks);
     HRESULT getGenericNetworkDrivers(std::vector<com::Utf8Str> &aGenericNetworkDrivers);
+    HRESULT getCloudNetworks(std::vector<ComPtr<ICloudNetwork> > &aCloudNetworks);
+    HRESULT getCloudProviderManager(ComPtr<ICloudProviderManager> &aCloudProviderManager);
 
    // wrapped IVirtualBox methods
     HRESULT composeMachineFilename(const com::Utf8Str &aName,
@@ -327,7 +368,8 @@ private:
     HRESULT createSharedFolder(const com::Utf8Str &aName,
                                const com::Utf8Str &aHostPath,
                                BOOL aWritable,
-                               BOOL aAutomount);
+                               BOOL aAutomount,
+                               const com::Utf8Str &aAutoMountPoint);
     HRESULT removeSharedFolder(const com::Utf8Str &aName);
     HRESULT getExtraDataKeys(std::vector<com::Utf8Str> &aKeys);
     HRESULT getExtraData(const com::Utf8Str &aKey,
@@ -345,16 +387,20 @@ private:
     HRESULT findNATNetworkByName(const com::Utf8Str &aNetworkName,
                                  ComPtr<INATNetwork> &aNetwork);
     HRESULT removeNATNetwork(const ComPtr<INATNetwork> &aNetwork);
+    HRESULT createCloudNetwork(const com::Utf8Str &aNetworkName,
+                               ComPtr<ICloudNetwork> &aNetwork);
+    HRESULT findCloudNetworkByName(const com::Utf8Str &aNetworkName,
+                                   ComPtr<ICloudNetwork> &aNetwork);
+    HRESULT removeCloudNetwork(const ComPtr<ICloudNetwork> &aNetwork);
     HRESULT checkFirmwarePresent(FirmwareType_T aFirmwareType,
                                  const com::Utf8Str &aVersion,
                                  com::Utf8Str &aUrl,
                                  com::Utf8Str &aFile,
                                  BOOL *aResult);
 
-    static HRESULT i_setErrorStatic(HRESULT aResultCode,
-                                    const Utf8Str &aText)
+    static HRESULT i_setErrorStaticBoth(HRESULT aResultCode, int vrc, const Utf8Str &aText)
     {
-        return setErrorInternal(aResultCode, getStaticClassIID(), getStaticComponentName(), aText, false, true);
+        return setErrorInternal(aResultCode, getStaticClassIID(), getStaticComponentName(), aText, false, true, vrc);
     }
 
     HRESULT i_registerMachine(Machine *aMachine);
@@ -400,9 +446,18 @@ private:
     static void i_SVCHelperClientThreadTask(StartSVCHelperClientData *pTask);
 #endif
 
+#if defined(RT_OS_WINDOWS) && defined(VBOXSVC_WITH_CLIENT_WATCHER)
+protected:
+    void i_callHook(const char *a_pszFunction) RT_OVERRIDE;
+    bool i_watchClientProcess(RTPROCESS a_pidClient, const char *a_pszFunction);
+public:
+    static void i_logCaller(const char *a_pszFormat, ...);
+private:
+
+#endif
 };
 
 ////////////////////////////////////////////////////////////////////////////////
 
-#endif // !____H_VIRTUALBOXIMPL
+#endif /* !MAIN_INCLUDED_VirtualBoxImpl_h */
 
