@@ -1,11 +1,10 @@
 /* $Id: ProgressImpl.cpp $ */
 /** @file
- *
  * VirtualBox Progress COM class implementation
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,9 +15,9 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
+#define LOG_GROUP LOG_GROUP_MAIN_PROGRESS
 #include <iprt/types.h>
 
-
 #if defined(VBOX_WITH_XPCOM)
 #include <nsIServiceManager.h>
 #include <nsIExceptionService.h>
@@ -32,17 +31,17 @@
 #endif
 #include "VirtualBoxErrorInfoImpl.h"
 
-#include "Logging.h"
-
 #include <iprt/time.h>
 #include <iprt/semaphore.h>
 #include <iprt/cpp/utils.h>
 
-#include <VBox/err.h>
-#include "AutoCaller.h"
+#include <iprt/errcore.h>
 
+#include "AutoCaller.h"
+#include "LoggingNew.h"
 #include "VBoxEvents.h"
 
+
 Progress::Progress()
 #if !defined(VBOX_COM_INPROC)
     : mParent(NULL)
@@ -147,11 +146,11 @@ HRESULT Progress::init(
                        VirtualBox *aParent,
 #endif
                        IUnknown *aInitiator,
-                       Utf8Str aDescription,
+                       const Utf8Str &aDescription,
                        BOOL aCancelable,
                        ULONG cOperations,
                        ULONG ulTotalOperationsWeight,
-                       Utf8Str aFirstOperationDescription,
+                       const Utf8Str &aFirstOperationDescription,
                        ULONG ulFirstOperationWeight)
 {
     LogFlowThisFunc(("aDescription=\"%s\", cOperations=%d, ulTotalOperationsWeight=%d, aFirstOperationDescription=\"%s\", ulFirstOperationWeight=%d\n",
@@ -167,19 +166,14 @@ HRESULT Progress::init(
     AutoInitSpan autoInitSpan(this);
     AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
-    HRESULT rc = S_OK;
-    rc = unconst(pEventSource).createObject();
-    if (FAILED(rc)) throw rc;
+    HRESULT rc = unconst(pEventSource).createObject();
+    if (FAILED(rc))
+        return rc;
 
     rc = pEventSource->init();
-    if (FAILED(rc)) throw rc;
+    if (FAILED(rc))
+        return rc;
 
-//    rc = Progress::init(
-//#if !defined(VBOX_COM_INPROC)
-//                        aParent,
-//#endif
-//                         aInitiator, aDescription, FALSE, aId);
-// NA
 #if !defined(VBOX_COM_INPROC)
     AssertReturn(aParent, E_INVALIDARG);
 #else
@@ -215,12 +209,6 @@ HRESULT Progress::init(
 
     unconst(mDescription) = aDescription;
 
-
-// end of assertion
-
-
-    if (FAILED(rc)) return rc;
-
     mCancelable = aCancelable;
 
     m_cOperations = cOperations;
@@ -236,11 +224,10 @@ HRESULT Progress::init(
 
     RTSemEventMultiReset(mCompletedSem);
 
-    /* Confirm a successful initialization when it's the case */
-    if (SUCCEEDED(rc))
-        autoInitSpan.setSucceeded();
+    /* Confirm a successful initialization. */
+    autoInitSpan.setSucceeded();
 
-    return rc;
+    return S_OK;
 }
 
 /**
@@ -260,7 +247,7 @@ HRESULT Progress::init(
  */
 HRESULT Progress::init(BOOL aCancelable,
                        ULONG aOperationCount,
-                       Utf8Str aOperationDescription)
+                       const Utf8Str &aOperationDescription)
 {
     LogFlowThisFunc(("aOperationDescription=\"%s\"\n", aOperationDescription.c_str()));
 
@@ -268,12 +255,6 @@ HRESULT Progress::init(BOOL aCancelable,
     AutoInitSpan autoInitSpan(this);
     AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
-    HRESULT rc = S_OK;
-    /* Guarantees subclasses call this method at the proper time */
-    NOREF(autoInitSpan);
-
-    if (FAILED(rc)) return rc;
-
     mCancelable = aCancelable;
 
     // for this variant we assume for now that all operations are weighed "1"
@@ -291,11 +272,10 @@ HRESULT Progress::init(BOOL aCancelable,
 
     RTSemEventMultiReset(mCompletedSem);
 
-    /* Confirm a successful initialization when it's the case */
-    if (SUCCEEDED(rc))
-        autoInitSpan.setSucceeded();
+    /* Confirm a successful initialization. */
+    autoInitSpan.setSucceeded();
 
-    return rc;
+    return S_OK;
 }
 
 
@@ -388,7 +368,7 @@ HRESULT Progress::i_notifyComplete(HRESULT aResultCode)
 #endif /* !defined(VBOX_WITH_XPCOM) */
     }
 
-    return i_notifyCompleteEI(aResultCode, errorInfo);
+    return notifyComplete(aResultCode, errorInfo);
 }
 
 /**
@@ -432,89 +412,54 @@ HRESULT Progress::i_notifyCompleteV(HRESULT aResultCode,
     AssertComRCReturnRC(rc);
     errorInfo->init(aResultCode, aIID, pcszComponent, text);
 
-    return i_notifyCompleteEI(aResultCode, errorInfo);
+    return notifyComplete(aResultCode, errorInfo);
+}
+
+/**
+ * Wrapper around Progress:notifyCompleteBothV.
+ */
+HRESULT Progress::i_notifyCompleteBoth(HRESULT aResultCode,
+                                       int vrc,
+                                       const GUID &aIID,
+                                       const char *pcszComponent,
+                                       const char *aText,
+                                       ...)
+{
+    va_list va;
+    va_start(va, aText);
+    HRESULT hrc = i_notifyCompleteBothV(aResultCode, vrc, aIID, pcszComponent, aText, va);
+    va_end(va);
+    return hrc;
 }
 
 /**
  * Marks the operation as complete and attaches full error info.
  *
- * This is where the actual work is done, the related methods all end up here.
- *
  * @param aResultCode   Operation result (error) code, must not be S_OK.
- * @param aErrorInfo            List of arguments for the format string.
+ * @param vrc           VBox status code to associate with the error.
+ * @param aIID          IID of the interface that defines the error.
+ * @param pszComponent  Name of the component that generates the error.
+ * @param pszFormat     Error message (must not be null), an RTStrPrintf-like
+ *                      format string in UTF-8 encoding.
+ * @param va            List of arguments for the format string.
  */
-HRESULT Progress::i_notifyCompleteEI(HRESULT aResultCode, const ComPtr<IVirtualBoxErrorInfo> &aErrorInfo)
+HRESULT Progress::i_notifyCompleteBothV(HRESULT aResultCode,
+                                        int vrc,
+                                        const GUID &aIID,
+                                        const char *pszComponent,
+                                        const char *pszFormat,
+                                        va_list va)
 {
-    LogThisFunc(("aResultCode=%d\n", aResultCode));
-    /* on failure we expect error info, on success there must be none */
-    AssertMsg(FAILED(aResultCode) ^ aErrorInfo.isNull(),
-              ("No error info but trying to set a failed result (%08X)!\n",
-               aResultCode));
+    /* expected to be used only in case of error */
+    Assert(FAILED(aResultCode));
 
-    AutoCaller autoCaller(this);
-    AssertComRCReturnRC(autoCaller.rc());
+    Utf8Str text(pszFormat, va);
+    ComObjPtr<VirtualBoxErrorInfo> errorInfo;
+    HRESULT rc = errorInfo.createObject();
+    AssertComRCReturnRC(rc);
+    errorInfo->initEx(aResultCode, vrc, aIID, pszComponent, text);
 
-    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-
-    AssertReturn(mCompleted == FALSE, E_FAIL);
-
-    if (mCanceled && SUCCEEDED(aResultCode))
-        aResultCode = E_FAIL;
-
-    mCompleted = TRUE;
-    mResultCode = aResultCode;
-    if (SUCCEEDED(aResultCode))
-    {
-        m_ulCurrentOperation = m_cOperations - 1; /* last operation */
-        m_ulOperationPercent = 100;
-    }
-    mErrorInfo = aErrorInfo;
-
-#if !defined VBOX_COM_INPROC
-    /* remove from the global collection of pending progress operations */
-    if (mParent)
-        mParent->i_removeProgress(mId.ref());
-#endif
-
-    /* wake up all waiting threads */
-    if (mWaitersCount > 0)
-        RTSemEventMultiSignal(mCompletedSem);
-
-    fireProgressTaskCompletedEvent(pEventSource, mId.toUtf16().raw());
-
-    return S_OK;
-}
-
-/**
- * Notify the progress object that we're almost at the point of no return.
- *
- * This atomically checks for and disables cancelation.  Calls to
- * IProgress::Cancel() made after a successful call to this method will fail
- * and the user can be told.  While this isn't entirely clean behavior, it
- * prevents issues with an irreversible actually operation succeeding while the
- * user believe it was rolled back.
- *
- * @returns Success indicator.
- * @retval  true on success.
- * @retval  false if the progress object has already been canceled or is in an
- *          invalid state
- */
-bool Progress::i_notifyPointOfNoReturn(void)
-{
-    AutoCaller autoCaller(this);
-    AssertComRCReturn(autoCaller.rc(), false);
-
-    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-
-    if (mCanceled)
-    {
-        LogThisFunc(("returns false\n"));
-        return false;
-    }
-
-    mCancelable = FALSE;
-    LogThisFunc(("returns true\n"));
-    return true;
+    return notifyComplete(aResultCode, errorInfo);
 }
 
 /**
@@ -584,6 +529,7 @@ bool Progress::i_setCancelCallback(void (*pfnCallback)(void *), void *pvUser)
     return i_iprtProgressCallback(uPercentage, pvUser);
 }
 
+
 // IProgress properties
 /////////////////////////////////////////////////////////////////////////////
 
@@ -707,8 +653,7 @@ HRESULT Progress::getResultCode(LONG *aResultCode)
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     if (!mCompleted)
-        return setError(E_FAIL,
-                        tr("Result code is not available, operation is still in progress"));
+        return setError(E_FAIL, tr("Result code is not available, operation is still in progress"));
 
     *aResultCode = mResultCode;
 
@@ -720,8 +665,7 @@ HRESULT Progress::getErrorInfo(ComPtr<IVirtualBoxErrorInfo> &aErrorInfo)
     AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     if (!mCompleted)
-        return setError(E_FAIL,
-                        tr("Error info is not available, operation is still in progress"));
+        return setError(E_FAIL, tr("Error info is not available, operation is still in progress"));
 
     mErrorInfo.queryInterfaceTo(aErrorInfo.asOutParam());
 
@@ -790,84 +734,23 @@ HRESULT Progress::setTimeout(ULONG aTimeout)
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     if (!mCancelable)
-        return setError(VBOX_E_INVALID_OBJECT_STATE,
-                        tr("Operation cannot be canceled"));
+        return setError(VBOX_E_INVALID_OBJECT_STATE, tr("Operation cannot be canceled"));
     m_cMsTimeout = aTimeout;
 
     return S_OK;
 }
 
+HRESULT Progress::getEventSource(ComPtr<IEventSource> &aEventSource)
+{
+    /* event source is const, no need to lock */
+    pEventSource.queryInterfaceTo(aEventSource.asOutParam());
+    return S_OK;
+}
+
 
 // IProgress methods
 /////////////////////////////////////////////////////////////////////////////
 
-/**
- * Updates the percentage value of the current operation.
- *
- * @param aPercent  New percentage value of the operation in progress
- *                  (in range [0, 100]).
- */
-HRESULT Progress::setCurrentOperationProgress(ULONG aPercent)
-{
-    AssertMsgReturn(aPercent <= 100, ("%u\n", aPercent), E_INVALIDARG);
-
-    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-
-    i_checkForAutomaticTimeout();
-    if (mCancelable && mCanceled)
-        AssertReturn(!mCompleted, E_FAIL);
-    AssertReturn(!mCompleted && !mCanceled, E_FAIL);
-
-    if (m_ulOperationPercent != aPercent)
-    {
-        m_ulOperationPercent = aPercent;
-        ULONG actualPercent = 0;
-        getPercent(&actualPercent);
-        fireProgressPercentageChangedEvent(pEventSource, mId.toUtf16().raw(), actualPercent);
-    }
-
-    return S_OK;
-}
-
-/**
- * Signals that the current operation is successfully completed and advances to
- * the next operation. The operation percentage is reset to 0.
- *
- * @param aNextOperationDescription  Description of the next operation.
- * @param aNextOperationsWeight     Weight of the next operation.
- *
- * @note The current operation must not be the last one.
- */
-HRESULT Progress::setNextOperation(const com::Utf8Str &aNextOperationDescription, ULONG aNextOperationsWeight)
-{
-    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
-
-    if (mCanceled)
-        return E_FAIL;
-    AssertReturn(!mCompleted, E_FAIL);
-    AssertReturn(m_ulCurrentOperation + 1 < m_cOperations, E_FAIL);
-
-    ++m_ulCurrentOperation;
-    m_ulOperationsCompletedWeight += m_ulCurrentOperationWeight;
-
-    m_operationDescription = aNextOperationDescription;
-    m_ulCurrentOperationWeight = aNextOperationsWeight;
-    m_ulOperationPercent = 0;
-
-    LogThisFunc(("%s: aNextOperationsWeight = %d; m_ulCurrentOperation is now %d, m_ulOperationsCompletedWeight is now %d\n",
-                 m_operationDescription.c_str(), aNextOperationsWeight, m_ulCurrentOperation, m_ulOperationsCompletedWeight));
-
-    /* wake up all waiting threads */
-    if (mWaitersCount > 0)
-        RTSemEventMultiSignal(mCompletedSem);
-
-    ULONG actualPercent = 0;
-    getPercent(&actualPercent);
-    fireProgressPercentageChangedEvent(pEventSource, mId.toUtf16().raw(), actualPercent);
-
-    return S_OK;
-}
-
 /**
  * @note XPCOM: when this method is not called on the main XPCOM thread, it
  *       simply blocks the thread until mCompletedSem is signalled. If the
@@ -914,9 +797,7 @@ HRESULT Progress::waitForCompletion(LONG aTimeout)
         }
 
         if (RT_FAILURE(vrc) && vrc != VERR_TIMEOUT)
-            return setError(VBOX_E_IPRT_ERROR,
-                            tr("Failed to wait for the task completion (%Rrc)"),
-                            vrc);
+            return setErrorBoth(VBOX_E_IPRT_ERROR, vrc, tr("Failed to wait for the task completion (%Rrc)"), vrc);
     }
 
     LogFlowThisFuncLeave();
@@ -976,9 +857,7 @@ HRESULT Progress::waitForOperationCompletion(ULONG aOperation, LONG aTimeout)
         }
 
         if (RT_FAILURE(vrc) && vrc != VERR_TIMEOUT)
-            return setError(E_FAIL,
-                            tr("Failed to wait for the operation completion (%Rrc)"),
-                            vrc);
+            return setErrorBoth(E_FAIL, vrc, tr("Failed to wait for the operation completion (%Rrc)"), vrc);
     }
 
     LogFlowThisFuncLeave();
@@ -986,11 +865,65 @@ HRESULT Progress::waitForOperationCompletion(ULONG aOperation, LONG aTimeout)
     return S_OK;
 }
 
-HRESULT Progress::waitForAsyncProgressCompletion(const ComPtr<IProgress> &aPProgressAsync)
+HRESULT Progress::cancel()
+{
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    if (!mCancelable)
+        return setError(VBOX_E_INVALID_OBJECT_STATE, tr("Operation cannot be canceled"));
+
+    if (!mCanceled)
+    {
+        LogThisFunc(("Canceling\n"));
+        mCanceled = TRUE;
+        if (m_pfnCancelCallback)
+            m_pfnCancelCallback(m_pvCancelUserArg);
+
+    }
+    else
+        LogThisFunc(("Already canceled\n"));
+
+    return S_OK;
+}
+
+
+// IInternalProgressControl methods
+/////////////////////////////////////////////////////////////////////////////
+
+/**
+ * Updates the percentage value of the current operation.
+ *
+ * @param aPercent  New percentage value of the operation in progress
+ *                  (in range [0, 100]).
+ */
+HRESULT Progress::setCurrentOperationProgress(ULONG aPercent)
+{
+    AssertMsgReturn(aPercent <= 100, ("%u\n", aPercent), E_INVALIDARG);
+
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    i_checkForAutomaticTimeout();
+    if (mCancelable && mCanceled)
+        AssertReturn(!mCompleted, E_FAIL);
+    AssertReturn(!mCompleted && !mCanceled, E_FAIL);
+
+    if (m_ulOperationPercent != aPercent)
+    {
+        m_ulOperationPercent = aPercent;
+        ULONG actualPercent = 0;
+        getPercent(&actualPercent);
+        fireProgressPercentageChangedEvent(pEventSource, mId.toUtf16().raw(), actualPercent);
+    }
+
+    return S_OK;
+}
+
+HRESULT Progress::waitForOtherProgressCompletion(const ComPtr<IProgress> &aProgressOther,
+                                                 ULONG aTimeoutMS)
 {
     LogFlowThisFuncEnter();
 
-    /* Note: we don't lock here, cause we just using public methods. */
+    /* Note: no locking needed, because we just use public methods. */
 
     HRESULT rc           = S_OK;
     BOOL fCancelable     = FALSE;
@@ -1000,10 +933,14 @@ HRESULT Progress::waitForAsyncProgressCompletion(const ComPtr<IProgress> &aPProg
     ULONG currentPercent = 0;
     ULONG cOp            = 0;
     /* Is the async process cancelable? */
-    rc = aPProgressAsync->COMGETTER(Cancelable)(&fCancelable);
+    rc = aProgressOther->COMGETTER(Cancelable)(&fCancelable);
     if (FAILED(rc)) return rc;
+
+    uint64_t u64StopTime = UINT64_MAX;
+    if (aTimeoutMS > 0)
+        u64StopTime = RTTimeMilliTS() + aTimeoutMS;
     /* Loop as long as the sync process isn't completed. */
-    while (SUCCEEDED(aPProgressAsync->COMGETTER(Completed(&fCompleted))))
+    while (SUCCEEDED(aProgressOther->COMGETTER(Completed(&fCompleted))))
     {
         /* We can forward any cancel request to the async process only when
          * it is cancelable. */
@@ -1013,7 +950,7 @@ HRESULT Progress::waitForAsyncProgressCompletion(const ComPtr<IProgress> &aPProg
             if (FAILED(rc)) return rc;
             if (fCanceled)
             {
-                rc = aPProgressAsync->Cancel();
+                rc = aProgressOther->Cancel();
                 if (FAILED(rc)) return rc;
             }
         }
@@ -1030,15 +967,15 @@ HRESULT Progress::waitForAsyncProgressCompletion(const ComPtr<IProgress> &aPProg
             ULONG curOp;
             for (;;)
             {
-                rc = aPProgressAsync->COMGETTER(Operation(&curOp));
+                rc = aProgressOther->COMGETTER(Operation(&curOp));
                 if (FAILED(rc)) return rc;
                 if (cOp != curOp)
                 {
                     Bstr bstr;
                     ULONG currentWeight;
-                    rc = aPProgressAsync->COMGETTER(OperationDescription(bstr.asOutParam()));
+                    rc = aProgressOther->COMGETTER(OperationDescription(bstr.asOutParam()));
                     if (FAILED(rc)) return rc;
-                    rc = aPProgressAsync->COMGETTER(OperationWeight(&currentWeight));
+                    rc = aProgressOther->COMGETTER(OperationWeight(&currentWeight));
                     if (FAILED(rc)) return rc;
                     rc = SetNextOperation(bstr.raw(), currentWeight);
                     if (FAILED(rc)) return rc;
@@ -1048,7 +985,7 @@ HRESULT Progress::waitForAsyncProgressCompletion(const ComPtr<IProgress> &aPProg
                     break;
             }
 
-            rc = aPProgressAsync->COMGETTER(OperationPercent(&currentPercent));
+            rc = aProgressOther->COMGETTER(OperationPercent(&currentPercent));
             if (FAILED(rc)) return rc;
             if (currentPercent != prevPercent)
             {
@@ -1060,45 +997,160 @@ HRESULT Progress::waitForAsyncProgressCompletion(const ComPtr<IProgress> &aPProg
         if (fCompleted)
             break;
 
-        /* Make sure the loop is not too tight */
-        rc = aPProgressAsync->WaitForCompletion(100);
-        if (FAILED(rc)) return rc;
+        if (aTimeoutMS != 0)
+        {
+            /* Make sure the loop is not too tight */
+            uint64_t u64Now = RTTimeMilliTS();
+            uint64_t u64RemainingMS = u64StopTime - u64Now;
+            if (u64RemainingMS < 10)
+                u64RemainingMS = 10;
+            else if (u64RemainingMS > 200)
+                u64RemainingMS = 200;
+            rc = aProgressOther->WaitForCompletion((LONG)u64RemainingMS);
+            if (FAILED(rc)) return rc;
+
+            if (RTTimeMilliTS() >= u64StopTime)
+                return VBOX_E_TIMEOUT;
+        }
+        else
+        {
+            /* Make sure the loop is not too tight */
+            rc = aProgressOther->WaitForCompletion(200);
+            if (FAILED(rc)) return rc;
+        }
+    }
+
+    /* Transfer error information if applicable and report the error status
+     * back to the caller to make this as easy as possible. */
+    LONG iRc;
+    rc = aProgressOther->COMGETTER(ResultCode)(&iRc);
+    if (FAILED(rc)) return rc;
+    if (FAILED(iRc))
+    {
+        setError(ProgressErrorInfo(aProgressOther));
+        rc = iRc;
     }
 
     LogFlowThisFuncLeave();
-
     return rc;
 }
 
-HRESULT Progress::cancel()
+/**
+ * Signals that the current operation is successfully completed and advances to
+ * the next operation. The operation percentage is reset to 0.
+ *
+ * @param aNextOperationDescription  Description of the next operation.
+ * @param aNextOperationsWeight     Weight of the next operation.
+ *
+ * @note The current operation must not be the last one.
+ */
+HRESULT Progress::setNextOperation(const com::Utf8Str &aNextOperationDescription, ULONG aNextOperationsWeight)
 {
     AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    if (!mCancelable)
-        return setError(VBOX_E_INVALID_OBJECT_STATE,
-                        tr("Operation cannot be canceled"));
+    if (mCanceled)
+        return E_FAIL;
+    AssertReturn(!mCompleted, E_FAIL);
+    AssertReturn(m_ulCurrentOperation + 1 < m_cOperations, E_FAIL);
 
-    if (!mCanceled)
-    {
-        LogThisFunc(("Canceling\n"));
-        mCanceled = TRUE;
-        if (m_pfnCancelCallback)
-            m_pfnCancelCallback(m_pvCancelUserArg);
+    ++m_ulCurrentOperation;
+    m_ulOperationsCompletedWeight += m_ulCurrentOperationWeight;
 
-    }
-    else
-        LogThisFunc(("Already canceled\n"));
+    m_operationDescription = aNextOperationDescription;
+    m_ulCurrentOperationWeight = aNextOperationsWeight;
+    m_ulOperationPercent = 0;
+
+    LogThisFunc(("%s: aNextOperationsWeight = %d; m_ulCurrentOperation is now %d, m_ulOperationsCompletedWeight is now %d\n",
+                 m_operationDescription.c_str(), aNextOperationsWeight, m_ulCurrentOperation, m_ulOperationsCompletedWeight));
+
+    /* wake up all waiting threads */
+    if (mWaitersCount > 0)
+        RTSemEventMultiSignal(mCompletedSem);
+
+    ULONG actualPercent = 0;
+    getPercent(&actualPercent);
+    fireProgressPercentageChangedEvent(pEventSource, mId.toUtf16().raw(), actualPercent);
 
     return S_OK;
 }
 
-HRESULT Progress::getEventSource(ComPtr<IEventSource> &aEventSource)
+/**
+ * Notify the progress object that we're almost at the point of no return.
+ *
+ * This atomically checks for and disables cancelation.  Calls to
+ * IProgress::Cancel() made after a successful call to this method will fail
+ * and the user can be told.  While this isn't entirely clean behavior, it
+ * prevents issues with an irreversible actually operation succeeding while the
+ * user believe it was rolled back.
+ *
+ * @returns COM error status.
+ * @retval  S_OK on success.
+ * @retval  E_FAIL if the progress object has already been canceled or is in an
+ *          invalid state
+ */
+HRESULT Progress::notifyPointOfNoReturn(void)
 {
-    /* event source is const, no need to lock */
-    pEventSource.queryInterfaceTo(aEventSource.asOutParam());
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    if (mCanceled)
+    {
+        LogThisFunc(("returns failure\n"));
+        return E_FAIL;
+    }
+
+    mCancelable = FALSE;
+    LogThisFunc(("returns success\n"));
     return S_OK;
 }
 
+/**
+ * Marks the operation as complete and attaches full error info.
+ *
+ * This is where the actual work is done, the related methods all end up here.
+ *
+ * @param aResultCode   Operation result (error) code, must not be S_OK.
+ * @param aErrorInfo            List of arguments for the format string.
+ */
+HRESULT Progress::notifyComplete(LONG aResultCode, const ComPtr<IVirtualBoxErrorInfo> &aErrorInfo)
+{
+    LogThisFunc(("aResultCode=%d\n", aResultCode));
+    /* on failure we expect error info, on success there must be none */
+    AssertMsg(FAILED(aResultCode) ^ aErrorInfo.isNull(),
+              ("No error info but trying to set a failed result (%08X)!\n",
+               aResultCode));
+
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
+
+    AssertReturn(mCompleted == FALSE, E_FAIL);
+
+    if (mCanceled && SUCCEEDED(aResultCode))
+        aResultCode = E_FAIL;
+
+    mCompleted = TRUE;
+    mResultCode = aResultCode;
+    if (SUCCEEDED(aResultCode))
+    {
+        m_ulCurrentOperation = m_cOperations - 1; /* last operation */
+        m_ulOperationPercent = 100;
+    }
+    mErrorInfo = aErrorInfo;
+
+#if !defined(VBOX_COM_INPROC)
+    /* remove from the global collection of pending progress operations */
+    if (mParent)
+        mParent->i_removeProgress(mId.ref());
+#endif
+
+    /* wake up all waiting threads */
+    if (mWaitersCount > 0)
+        RTSemEventMultiSignal(mCompletedSem);
+
+    fireProgressTaskCompletedEvent(pEventSource, mId.toUtf16().raw());
+
+    return S_OK;
+}
+
+
 // private internal helpers
 /////////////////////////////////////////////////////////////////////////////
 
